<?php
/**
* Modelo Firewalllists para el Componente Securitycheckpro
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Chequeamos si el archivo est includo en Joomla!
defined('_JEXEC') or die();
jimport('joomla.html.pagination');

/**
* Modelo Securitycheck
*/
class SecuritycheckprosModelFirewallLists extends SecuritycheckproModel
{

/* Funcin que filtra un array segn un parmetro de bsqueda y paginacin */
function filter_data($array_data,&$pagination)
{
	/* Inicializamos las variables */
	$this->total = 0;
	
	/* Obtenemos el nmero de registros del array que hemos de mostrar. Si el lmite superior es '0', entonces devolvemos todo el array */
	$upper_limit = $this->getState('limitstart');
	$lower_limit = $this->getState('limit');
	
	/* Obtenemos los valores de los filtros */
	$search = htmlentities($this->state->get('filter.lists_search'));
	
	/* Nmero total de elementos en el array (necesario para la paginacin) */
	$this->total = count($array_data);
		
	$filtered_array = array();
	/* Si el campo 'search' no est vaco, buscamos en todos los campos del array */			
	if (!empty($search) ) {
		$filtered_array = array_values(array_filter($array_data, function ($element) use ($search) { return (strstr($element,$search) );} ));
		/* Cortamos el array para mostrar slo los valores mostrados por la paginacin */
		$array_data = array_splice($filtered_array, $upper_limit, $lower_limit);
	} else {
		$array_data = array_splice($array_data, $upper_limit, $lower_limit);
		
	}
	$pagination = new JPagination($this->total, $upper_limit, $lower_limit);
	sort($array_data,SORT_NUMERIC);
	return ($array_data);
}

/* Funcin que extrae las entradas de la BBDD '#__securitycheckpro_dynamic_blacklist' */
function get_dynamic_blacklist_ips() {
	
	// Inicializamos las variables
	$query = null;
	$db = JFactory::getDBO();
	$blacklist_ips = array();
		
	// Obtenemos el 'extension_id' del Firewall Web, disponible en la tabla '#__extensions'
	$query = $db->getQuery(true)
		->select(array($db->quoteName('ip')))
		->from($db->quoteName('#__securitycheckpro_dynamic_blacklist'));
	$db->setQuery($query);
	$blacklist_ips = $db->loadObjectList();
	if ( count($blacklist_ips)>0 ) {
		$blacklist_ips = get_object_vars($blacklist_ips[0]);
	}
	
	return $blacklist_ips;
}

/* Funcin que elimina IPs de la lista negra dinmica */
function deleteip_dynamic_blacklist() {
	// Inicializamos las variables
	$deleted_elements = 0;
	$db = JFactory::getDBO();
	
	// Creamos el objeto JInput para obtener las variables del formulario
	$jinput = JFactory::getApplication()->input;
	
	// Obtenemos los valores de las IPs que sern eliminados de la lista negra dinmica
	$uids = $jinput->get('dynamic_blacklist_table','0','array');
		
	foreach($uids as $uid) {
		// IP sanitizada
		$ip_to_delete = $db->Quote($db->escape($uid));
		// Borramos la IP de la tabla
		$query = "DELETE FROM `#__securitycheckpro_dynamic_blacklist` WHERE (ip = {$ip_to_delete})";
		$db->setQuery( $query );
		$result = $db->query();
		if ( $result ) {
			$deleted_elements++;
		}		
	}
	JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_DELETED_FROM_LIST',$deleted_elements));
}

/* Funcin para aadir una ip a una lista */
function manage_list($type,$action){

	// Inicializamos las variables
	$query = null;
	$array_size = 0;
	$added_elements = 0;
	$deleted_elements = 0;
	$ip_to_add = null;
	$uids = null;
			
	$db = JFactory::getDBO();
	
	// Creamos el objeto JInput para obtener las variables del formulario
	$jinput = JFactory::getApplication()->input;
	
	// Obtenemos la configuracin del plugin
	$params = $this->getConfig();
	
	switch ($action) {
		case "add":
			// Obtenemos el valor de la IP introducida
			if ( $type == 'blacklist' ) {
				$ip_to_add = $jinput->get('blacklist_add_ip','0.0.0.0','string');
			} else if ( $type == 'whitelist' ) {
				$ip_to_add = $jinput->get('whitelist_add_ip','0.0.0.0','string');
			}
			// Chequeamos el formato de la entrada
			if ( strstr($ip_to_add,'*') ) { // Si existe algn comodn, lo reemplazamos por el dgito '0'
				$ip_to_add_filtered= str_replace('*','0',$ip_to_add);
				$ip_valid = filter_var($ip_to_add_filtered,FILTER_VALIDATE_IP,FILTER_FLAG_IPV4);
			} else {
				$ip_valid = filter_var($ip_to_add,FILTER_VALIDATE_IP,FILTER_FLAG_IPV4);
			}
			
			if ( !$ip_valid ) {
				JError::raiseWarning(100,JText::_('COM_SECURITYCHECKPRO_INVALID_FORMAT'));
				break;
			}
			
			$aparece_lista = $this->chequear_ip_en_lista($ip_to_add,$params[$type]);
			if (!$aparece_lista) {
				if ( $params[$type] != '' ) {
					$params[$type] .= ',' .$ip_to_add;
				} else {
					$params[$type] .= $ip_to_add;
				}
				$added_elements++;
			}
			
			if ($added_elements > 0) {
				JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_ADDED_TO_LIST',$added_elements));
			} else {
				JError::raiseNotice(100,JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_IGNORED',1));
			}
			break;
		case "delete":
			// Obtenemos los valores de las IPs que sern introducidas en la lista negra
			if ( $type == 'blacklist' ) {
				$uids = $jinput->get('cid','0','array');
			} else if ( $type == 'whitelist' ) {
				$uids = $jinput->get('whitelist_cid','0','array');
			}
						
			// Obtenemos los valores que ya existen en la lista negra
			$list_to_array = explode(',',$params[$type]);
			$list_to_array = array_map( function ($element) { return trim($element); },$list_to_array );
			if ( $uids != 0 ) {
				foreach($uids as $uid) {
					$key = array_search($uid,$list_to_array);
					if ( $key !== false ) {
						// Eliminamos el elemento del array
						array_splice($list_to_array, $key, 1);
						$deleted_elements++;
					}
				}
				if ($deleted_elements > 0) {
					JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_DELETED_FROM_LIST',$deleted_elements));
					sort($list_to_array,SORT_NUMERIC);
					$params[$type] = implode(',',$list_to_array);
				}
			}
			break;
	}
	$this->saveConfig($params,'pro_plugin');
}

/* Funcin que obtiene la geolocalizacin de una ip */
function geolocation($ip) {

	$db = JFactory::getDBO();
	
	/* Cargamos el lenguaje del sitio */
	$lang = JFactory::getLanguage();
	$lang->load('com_securitycheckpro',JPATH_ADMINISTRATOR);
			
	// Geolocalizacin de la IP
	$country_name = "";
	$continent_name = "";
			
	if(@file_exists(JPATH_ADMINISTRATOR.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_securitycheckpro'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'GeoIP.dat')) {
		/* Chequeamos si existen las funciones necesarias para manejar el fichero de geolocalizacin. Si cargamos nuestro fichero y estas
			funciones ya estn definidas, obtendremos un error fatal */			
		if ( !function_exists('geoip_open') ) {
			require_once JPATH_ADMINISTRATOR.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_securitycheckpro'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'geoip.php';
		}
		require_once JPATH_ADMINISTRATOR.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_securitycheckpro'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'geoblock.php';
		$gi = geoip_open(JPATH_ADMINISTRATOR.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_securitycheckpro'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'GeoIP.dat',GEOIP_STANDARD);
						
		// Obtenemos los cdigos de pas y continente y el nombre del pas
		$country = geoip_country_code_by_addr_sc($gi, $ip);
		$continent = getContinent($country);		
	} else {
		$country = '(unknown country)';
		$continent = '(unknown continent)';
	}
						
	// Obtenemos el nombre del continente y del pas
	$continent_name = getContinentName($continent);
	$country_name = geoip_country_name_by_addr_sc($gi, $ip);
			
	// Construimos el contenido del campo 'geolocation' que ser mostrado
	$geolocation = $lang->_('COM_SECURITYCHECKPRO_COUNTRY_LABEL') . ': ' . $db->escape($country_name) . ' | ' . $lang->_('COM_SECURITYCHECKPRO_CONTINENT_LABEL') . ': ' . $db->escape($continent_name);
			
	// Cerramos el manejador
	geoip_close($gi);
	
	// Devolvemos el resultado
	return $geolocation;
}

/* Funcin que cambia los wildcards para extraer la geolocalizacin */
function change_wildcards($ip){
	$ip_without_wildcards = '';
	$array_ip_peticionaria = explode('.',$ip); // Formato array:  $array_ip_lista[0] = '192' , $array_ip_lista[1] = '168'
		
	if (strrchr($ip,'*')){ // Chequeamos si existe el carcter '*' en el string; si no existe podemos ignorar esta ip
		$k = 0;
		while ( $k <= 3 )  {
			if ($array_ip_peticionaria[$k] == '*') {
				$array_ip_peticionaria[$k] = 0;					
			}
			$k++;
		}
		$ip_without_wildcards = implode('.',$array_ip_peticionaria);
		return $ip_without_wildcards;
	} else {
		return $ip;
	}
		
}

}