<?php
/**
* Modelo Logs para el Componente Securitycheckpro
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Chequeamos si el archivo est includo en Joomla!
defined('_JEXEC') or die();
jimport( 'joomla.application.component.model' );
jimport( 'joomla.access.rule' );
/**
* Modelo Vulninfo
*/
class SecuritycheckprosModelLogs extends JModelLegacy
{
/**
* Array de datos
* @var array
*/
var $_data;
/**
/**
* Total items
* @var integer
*/
var $_total = null;
/**
* Objeto Pagination
* @var object
*/
var $_pagination = null;

private $defaultConfig = array(
	'logs_attacks'			=> 1,	
);

function __construct()
{
	parent::__construct();
	
	
	$mainframe = JFactory::getApplication();
	
	// Obtenemos las variables de paginacin de la peticin
	$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
	$limitstart = JRequest::getVar('limitstart', 0, '', 'int');

	// En el caso de que los lmites hayan cambiado, los volvemos a ajustar
	$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

	$this->setState('limit', $limit);
	$this->setState('limitstart', $limitstart);
	
}

/***/
protected function populateState()
{
	// Inicializamos las variables
	$app		= JFactory::getApplication();
	
	$search = $app->getUserStateFromRequest('filter.search', 'filter_search');
	$this->setState('filter.search', $search);
	$description = $app->getUserStateFromRequest('filter.description', 'filter_description');
	$this->setState('filter.description', $description);
	$type = $app->getUserStateFromRequest('filter.type', 'filter_type');
	$this->setState('filter.type', $type);
	$leido = $app->getUserStateFromRequest('filter.leido', 'filter_leido');
	$this->setState('filter.leido', $leido);
	$datefrom = $app->getUserStateFromRequest('datefrom', 'datefrom');
	$this->setState('datefrom', $datefrom);
	$dateto = $app->getUserStateFromRequest('dateto', 'dateto');
	$this->setState('dateto', $dateto);
		
	parent::populateState();
}


/* 
* Funcin para obtener el nmero de registros de la BBDD 'securitycheckpro_logs'
*/
function getTotal()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_total)) {
	$query = $this->_buildQuery();
	$this->_total = $this->_getListCount($query);
}
return $this->_total;
}

/* 
* Funcin para obtener el nmero de registros de la BBDD 'securitycheckpro_logs' segn la opcin escogida por el usuario
*/
function getFilterTotal()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_total)) {
	$query = $this->_buildFilterQuery();
	$this->_total = $this->_getListCount($query);
}
return $this->_total;
}

/* 
* Funcin para la paginacin 
*/
function getPagination()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_pagination)) {
	jimport('joomla.html.pagination');
$this->_pagination = new JPagination($this->getTotal(), $this->getState('limitstart'), $this->getState('limit') );
}
return $this->_pagination;
}

/* 
* Funcin para la paginacin filtrada segn la opcin escogida por el usuario
*/
function getFilterPagination()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_pagination)) {
	jimport('joomla.html.pagination');
$this->_pagination = new JPagination($this->getFilterTotal(), $this->getState('limitstart'), $this->getState('limit') );
}
return $this->_pagination;
}

/*
* Devuelve todos los componentes almacenados en la BBDD 'securitycheckpro_logs'
*/
function _buildQuery()
{
$query = ' SELECT * '
. ' FROM #__securitycheckpro_logs ORDER BY id DESC'
;
return $query;
}

/*
* Devuelve todos los componentes almacenados en la BBDD 'securitycheckpro_logs' filtrados segn las opciones establecidas por el usuario
*/
function _buildFilterQuery()
{
	// Creamos el nuevo objeto query
	$db = $this->getDbo();
	$query = $db->getQuery(true);
	
	// Sanitizamos la entrada
	$search = $this->state->get('filter.search');
	$search = $db->Quote('%'.$db->escape($search, true).'%');
		
	$query->select('*');
	$query->from('#__securitycheckpro_logs AS a');
	$query->where('(a.ip LIKE '.$search.' OR a.time LIKE '.$search.' OR a.description LIKE '.$search.' OR a.uri LIKE '.$search.')');
	
	// Filtramos la descripcion
	if ($description = $this->getState('filter.description')) {
		$query->where('a.tag_description = '.$db->quote($description));
	}
	
	// Filtramos el tipo
	if ($log_type = $this->getState('filter.type')) {
		$query->where('a.type = '.$db->quote($log_type));
	}
		
	// Filtramos leido/no leido
	$leido = $this->getState('filter.leido');
	if (is_numeric($leido)) {
		$query->where('a.marked = '.(int) $leido);
	}	
	
	
	// Filtramos el rango de fechas
	JLoader::import('joomla.utilities.date');

	$fltDateFrom = $this->getState('datefrom', null, 'string');
	if($fltDateFrom) {
		$regex = '/^\d{1,4}(\/|-)\d{1,2}(\/|-)\d{2,4}[[:space:]]{0,}(\d{1,2}:\d{1,2}(:\d{1,2}){0,1}){0,1}$/';
		if(!preg_match($regex, $fltDateFrom)) {
			$fltDateFrom = '2000-01-01 00:00:00';
			$this->setState('datefrom', '');
		}
		$date = new JDate($fltDateFrom);
		$query->where($db->quoteName('time').' >= '.$db->Quote($date->toSql()));
	}

	$fltDateTo = $this->getState('dateto', null, 'string');
	if($fltDateTo) {
		$regex = '/^\d{1,4}(\/|-)\d{1,2}(\/|-)\d{2,4}[[:space:]]{0,}(\d{1,2}:\d{1,2}(:\d{1,2}){0,1}){0,1}$/';
		if(!preg_match($regex, $fltDateTo)) {
			$fltDateTo = '2037-01-01 00:00:00';
			$this->setState('dateto', '');
		}
		$date = new JDate($fltDateTo);
		$query->where($db->quoteName('time').' <= '.$db->Quote($date->toSql()));
	}
	
	// Ordenamos el resultado
	$query = $query . ' ORDER BY a.id DESC';
		
return $query;
}

/**
 * Mtodo para cargar todas las vulnerabilidades de los componentes
 */
function getData()
{
	// Cargamos los datos
	if (empty( $this->_data )) {
		$query = $this->_buildQuery();
		$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
	}
		
	return $this->_data;
}

/**
 * Mtodo para cargar todas las vulnerabilidades de los componentes especificadas en los trminos de bsqueda
 */
function getFilterData()
{
	// Cargamos los datos
	if (empty( $this->_data )) {
		$query = $this->_buildFilterQuery();
		$this->_data = $this->_getList($query, $this->getState('limitstart'), $this->getState('limit'));
	}
			
	return $this->_data;
}

/* Funcin para cambiar el estado de un array de logs de no ledo a ledo */
function mark_read(){
	$uids = JRequest::getVar('cid', 0, '', 'array');
	
	JArrayHelper::toInteger($uids, array());
	
	$db = $this->getDbo();
	foreach($uids as $uid) {
		$sql = "UPDATE `#__securitycheckpro_logs` SET marked=1 WHERE id='{$uid}'";
		$db->setQuery($sql);
		$db->query();	
	}
}

/* Funcin para cambiar el estado de un array de logs de ledo a no ledo */
function mark_unread(){
	$uids = JRequest::getVar('cid', 0, '', 'array');
	
	JArrayHelper::toInteger($uids, array());
	
	$db = $this->getDbo();
	foreach($uids as $uid) {
		$sql = "UPDATE `#__securitycheckpro_logs` SET marked=0 WHERE id='{$uid}'";
		$db->setQuery($sql);
		$db->query();	
	}
}

/* Funcin para borrar un array de logs */
function delete(){
	$uids = JRequest::getVar('cid', 0, '', 'array');
	
	JArrayHelper::toInteger($uids, array());
	
	$db = $this->getDbo();
	foreach($uids as $uid) {
		$sql = "DELETE FROM `#__securitycheckpro_logs` WHERE id='{$uid}'";
		$db->setQuery($sql);
		$db->query();	
	}
}

/* Funcin para chequear si una ip pertenece a una lista en la que podemos especificar rangos. Podemos tener una ip del tipo 192.168.*.* y una ip 192.168.1.1 entrara en ese rango */
function chequear_ip_en_lista($ip,$lista){
	$aparece = false;
	$array_ip_peticionaria = explode('.',$ip);
		
	if (strlen($lista) > 0) {
		// Eliminamos los caracteres en blanco antes de introducir los valores en el array
		$lista = str_replace(' ','',$lista);
		$array_ips = explode(',',$lista);
		if ( is_int(array_search($ip,$array_ips)) ){	// La ip aparece tal cual en la lista
			$aparece = true;
		} else {
			foreach ($array_ips as &$indice){
					if (strrchr($indice,'*')){ // Chequeamos si existe el carcter '*' en el string; si no existe podemos ignorar esta ip
					$array_ip_lista = explode('.',$indice); // Formato array:  $array_ip_lista[0] = '192' , $array_ip_lista[1] = '168'
					$k = 0;
					$igual = true;
					while (($k <= 3) && ($igual)) {
						if ($array_ip_lista[$k] == '*') {
							$k++;
						}else {
							if ($array_ip_lista[$k] == $array_ip_peticionaria[$k]) {
								$k++;
							} else {
								$igual = false;
							}
						}
					}
					if ($igual) { // $igual ser true cuando hayamos recorrido el array y todas las partes del mismo coincidan con la ip que realiza la peticin
						$aparece = true;
						return $aparece;
					}
				}
			}
		}
	}
	return $aparece;
}

/* Funcin que aade un conjunto de Ips a la lista negra */
function add_to_blacklist() {
	
	// Inicializamos las variables
	$query = null;
	$array_size = 0;
	$added_elements = 0;
		
	$db = JFactory::getDBO();
	
	// Obtenemos los valores de las IPs que sern introducidas en la lista negra
	$uids = JRequest::getVar('cid', 0, '', 'array');
	JArrayHelper::toInteger($uids, array());
	
	// Nmero de elementos del array
	$array_size = count($uids);
		
	// Obtenemos los valores de las distintas opciones del Firewall Web
	$db = $this->getDbo();
	$query = $db->getQuery(true)
		->select(array($db->quoteName('storage_value')))
		->from($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote('pro_plugin'));
	$db->setQuery($query);
	$params = $db->loadResult();
	$params = json_decode($params, true);
		
	foreach($uids as $uid) {
		$sql = "SELECT ip FROM `#__securitycheckpro_logs` WHERE id='{$uid}'";
		$db->setQuery($sql);
		$db->query();
		$ip = $db->loadResult();
		$aparece_lista_negra = $this->chequear_ip_en_lista($ip,$params['blacklist']);
		if (!$aparece_lista_negra) {
			$params['blacklist'] .= ',' .$ip;
			$added_elements++;
		}
	}
	$not_added = $array_size - $added_elements;
	
	if ($added_elements > 0) {
		JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_ADDED_TO_LIST',$added_elements));
	}
	if ($not_added > 0) {
		JError::raiseNotice(100,JText::sprintf('COM_SECURITYCHECKPRO_ELEMENTS_IGNORED',$not_added));
	}
	
	// Codificamos de nuevo los parmetros y los introducimos en la BBDD
	$params = utf8_encode(json_encode($params));
		
	$query = $db->getQuery(true)
		->delete($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote('pro_plugin'));
	$db->setQuery($query);
	$db->query();
		
	$object = (object)array(
		'storage_key'		=> 'pro_plugin',
		'storage_value'		=> $params
	);
		
	try {
		$result = $db->insertObject('#__securitycheckpro_storage', $object);			
	} catch (Exception $e) {	
		$applied = false;
	}
		
}

/* Obtiene el valor de una opcin de configuracin */
public function getValue($key, $default = null, $key_name = 'cparams')
{
	if(is_null($this->config)) $this->load($key_name);
	
	if(version_compare(JVERSION, '3.0', 'ge')) {
		return $this->config->get($key, $default);
	} else {
		return $this->config->getValue($key, $default);
	}
}

/* Hace una consulta a la tabla especificada como parmetro  */
public function load($key_name)
{
	$db = JFactory::getDBO();
	$query = $db->getQuery(true);
	$query 
		->select($db->quoteName('storage_value'))
		->from($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote($key_name));
	$db->setQuery($query);
	$res = $db->loadResult();
		
	if(version_compare(JVERSION, '3.0', 'ge')) {
		$this->config = new JRegistry();
	} else {
		$this->config = new JRegistry('securitycheckpro');
	}
	if(!empty($res)) {
		$res = json_decode($res, true);
		$this->config->loadArray($res);
	}
}

/* Obtiene la configuracin de los parmetros de la opcin 'Mode' */
function getConfig()
{
	if(interface_exists('JModel')) {
		$params = JModelLegacy::getInstance('FirewallCPanel','SecuritycheckProsModel');
	} else {
		$params = JModel::getInstance('FirewallCPanel','SecuritycheckProsModel');
	}
	
	$config = array();
	foreach($this->defaultConfig as $k => $v) {
		$config[$k] = $params->getValue($k, $v, 'pro_plugin');
	}
	return $config;
}

}