<?php
/**
* Modelo Protection para el Componente Securitycheckpro
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Chequeamos si el archivo est includo en Joomla!
defined('_JEXEC') or die();
jimport('joomla.application.component.model');
jimport('joomla.filesystem.file');

if(!class_exists('JoomlaCompatModel')) {
	if(interface_exists('JModel')) {
		abstract class JoomlaCompatModel extends JModelLegacy {}
	} else {
		class JoomlaCompatModel extends JModel {}
	}
}

/**
* Modelo Securitycheck
*/
class SecuritycheckprosModelProtection extends JoomlaCompatModel
{

/* Definimos las variables */
var $defaultConfig = array(
	'disable_server_signature'	=> 0,
	'prevent_access'	=> 0,
	'prevent_unauthorized_browsing'	=> 0,
	'file_injection_protection'	=> 0,
	'self_environ'	=> 0,
	'xframe_options'	=>	0,
	'prevent_mime_attacks'	=>	0,
	'default_banned_list'	=> 0,
	'own_banned_list'	=> '',
	'disallow_php_eggs'	=> 0,
	'disallow_sensible_files_access' => 0,
	'hide_backend_url' => '',
);

var $ConfigApplied = array(
	'disable_server_signature'	=> 0,
	'prevent_access'	=> 0,
	'prevent_unauthorized_browsing'	=> 0,
	'file_injection_protection'	=> 0,
	'self_environ'	=> 0,
	'xframe_options'	=>	0,
	'prevent_mime_attacks'	=>	0,
	'default_banned_list'	=> 0,
	'own_banned_list'	=> 0,
	'disallow_php_eggs'	=> 0,
	'disallow_sensible_files_access' => 0,
	'hide_backend_url' => 0,
);

private $config = null;

/* Obtiene el valor de una opcin de configuracin de 'htaccess protection' */
public function getValue($key, $default = null, $key_name = 'cparams')
{
	if(is_null($this->config)) $this->load($key_name);
	
	if(version_compare(JVERSION, '3.0', 'ge')) {
		return $this->config->get($key, $default);
	} else {
		return $this->config->getValue($key, $default);
	}
}

/* Establece el valor de una opcin de configuracin de 'htaccess protection' */
public function setValue($key, $value, $save = false, $key_name = 'cparams')
{
	if(is_null($this->config)) {
		$this->load($key_name);
	}
		
	if(version_compare(JVERSION, '3.0', 'ge')) {
		$x = $this->config->set($key, $value);
	} else {
		$x = $this->config->setValue($key, $value);
	}
	if($save) $this->save($key_name);
	return $x;
}

/* Hace una consulta a la tabla #__securitycheckpro_storage, que contiene la configuracin de 'htaccess protection' */
public function load($key_name)
{
	$db = JFactory::getDBO();
	$query = $db->getQuery(true);
	$query 
		->select($db->quoteName('storage_value'))
		->from($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote($key_name));
	$db->setQuery($query);
	$res = $db->loadResult();
		
	if(version_compare(JVERSION, '3.0', 'ge')) {
		$this->config = new JRegistry();
	} else {
		$this->config = new JRegistry('securitycheckpro');
	}
	if(!empty($res)) {
		$res = json_decode($res, true);
		$this->config->loadArray($res);
	}
}

/* Guarda la configuracin de 'htaccess protection' con a la tabla #__securitycheckpro_storage */
public function save($key_name)
{
	if(is_null($this->config)) {
		$this->load($key_name);
	}
		
	$db = JFactory::getDBO();
	$query = $db->getQuery(true);
	
	$data = $this->config->toArray();
	$data = json_encode($data);
		
	$query
		->delete($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote($key_name));
	$db->setQuery($query);
	$db->query();
		
	$object = (object)array(
		'storage_key'		=> $key_name,
		'storage_value'		=> $data
	);
	$db->insertObject('#__securitycheckpro_storage', $object);
}

/* Obtiene la configuracin de los parmetros de la opcin 'Protection' */
function getConfig()
{
	$config = array();
	foreach($this->defaultConfig as $k => $v) {
		$config[$k] = $this->getValue($k, $v);
	}
	return $config;
}

/* Guarda la modificacin de los parmetros de la opcin 'Protection' */
function saveConfig($newParams, $key_name = 'cparams')
{
	foreach($newParams as $key => $value)
	{
		$this->setValue($key,$value,'',$key_name);
	}

	$this->save($key_name);
}

/* Devuelve TRUE si el fichero pasado como argumento existe en la raz del sitio. */
public function ExistsFile($filename)
{
	return JFile::exists(JPATH_SITE.DIRECTORY_SEPARATOR.$filename);
}

/* Hace una copia del archivo .htaccess si existe*/
function Make_Backup()
{
	return JFile::copy(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess',JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess.backup');
}

/* Modificamos los valores del array 'ConfigApplied' segnn las opciones que ya hayan sido aplicadas al archivo .htaccess existentes */
public function GetConfigApplied(){
	/* Variable que almacenar el contenido del archivo .htaccess */
	$rules_applied = null;
	/* Variable que indicar si existe el/los strings en el archivo .htaccess */
	$exists = false;
	
	if ( JFile::exists(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess') ) {
		$rules_applied = JFile::read(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess');
		
		/* 'prevent_access' habilitado? */
		if ( stripos($rules_applied,"<FilesMatch \"^.ht\">") ) {
			$this->ConfigApplied['prevent_access'] = 1;
		}
		/* 'prevent_unauthorized_browsing' habilitado? */
		if ( stripos($rules_applied,"Options All -Indexes") ) {
			$this->ConfigApplied['prevent_unauthorized_browsing'] = 1;
		}
		/* 'default_banned_list' habilitado? */
		if ( stripos($rules_applied,"## Begin Securitycheck Pro Default Blacklist") ) {
			$this->ConfigApplied['default_banned_list'] = 1;
		}
		/* 'file_injection_protection' habilitado? */
		if ( stripos($rules_applied,"RewriteCond %{REQUEST_METHOD} GET") ) {
			$this->ConfigApplied['file_injection_protection'] = 1;
		}
		/* 'self_environ' habilitado? */
		if ( stripos($rules_applied,"RewriteCond %{QUERY_STRING} proc/self/environ [NC,OR]") ) {
			$this->ConfigApplied['self_environ'] = 1;
		}
		/* 'xframe_options' habilitado? */
		if ( stripos($rules_applied,"Header set X-Frame-Options") ) {
			$this->ConfigApplied['xframe_options'] = 1;
		}
		/* 'prevent_mime_attacks' habilitado? */
		if ( stripos($rules_applied,'Header set X-Content-Type-Options "nosniff"') ) {
			$this->ConfigApplied['prevent_mime_attacks'] = 1;
		}
		/* 'own_banned_list' habilitado? */
		$current_own_banned_list = explode( PHP_EOL, $this->getValue("own_banned_list") );
		if ( ! empty( $current_own_banned_list ) && ! ( sizeof( $current_own_banned_list ) == 1 && trim( $current_own_banned_list[0] ) == ''  ) ) {
			$exists = true;
			foreach ( $current_own_banned_list as $agent_applied ) {
				$search_string = null;
				$search_string .= "RewriteCond %{HTTP_USER_AGENT} " . trim( $agent_applied );
				if ( !stripos($rules_applied,$search_string) ) {
					/* Si no existe el string, actualizamos la variable '$exists' y salimos del bucle 'foreach' */
					$exists = false;
					break;
				}
			}			
		}
		if ( $exists ) {
			$this->ConfigApplied['own_banned_list'] = 1;
		}
		/* 'disable_server_signature' habilitado? */
		if ( stripos($rules_applied,"ServerSignature Off") ) {
			$this->ConfigApplied['disable_server_signature'] = 1;
		}
		/* 'disallow_php_eggs' habilitado? */
		if ( stripos($rules_applied,"RewriteCond %{QUERY_STRING} \=PHP[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12} [NC]") ) {
			$this->ConfigApplied['disallow_php_eggs'] = 1;
		}
		/* 'disallow_sensible_files_access' habilitado? */
		if ( stripos($rules_applied,"RewriteRule ^(htaccess.txt|configuration.php(-dist)?|joomla.xml|README.txt|web.config.txt|CONTRIBUTING.md|phpunit.xml.dist)$ - [F]") ) {
			$this->ConfigApplied['disallow_sensible_files_access'] = 1;
		}
		/* 'hide_backend_url' habilitado? */
		if ( stripos($rules_applied,"RewriteCond %{QUERY_STRING} !^" . $this->getValue("hide_backend_url")) ) {
			$this->ConfigApplied['hide_backend_url'] = 1;
		}
	}
	
	return $this->ConfigApplied;
}

/* Modifica o crea el archivo .htaccess segn las opciones escogidas por el usuario */
public function protect()
{
	// Site's url
	$site_url = str_replace('http://',"",JURI::base());
	
	$rules = null;
			
	$ExistsHtaccess = $this->ExistsFile('.htaccess');  // Comprobamos si existe el archivo .htaccess
	if ( $ExistsHtaccess) {  // Si existe, hacemos un backup
		$backup_sucess = $this->Make_Backup();
		// Leemos el contenido del fichero .htaccess existente y lo guardamos en el buffer.
		$rules .= JFile::read(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess');
		// Obtenemos los valores que ya estn aplicados para evitar duplicar valores
		$this-> ConfigApplied = $this->GetConfigApplied();
	} else {  /* Si no existe el fichero, copiamos el que incorpora Joomla por defecto */
		// Leemos el contenido del fichero .htaccess existente y lo guardamos en el buffer.
		$rules .= JFile::read(JPATH_COMPONENT_ADMINISTRATOR.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'default_joomla_htaccess.inc');
		$status = JFile::copy(JPATH_COMPONENT_ADMINISTRATOR.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'default_joomla_htaccess.inc',JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess');
	}
	
	/* Comprobamos si hay que proteger los archivos .ht */
	if ( $this->getValue("prevent_access") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['prevent_access'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro Prevent access to .ht files";
			$rules .= PHP_EOL . "<FilesMatch \"^.ht\">";
			$rules .= PHP_EOL . "Order deny,allow";
			$rules .= PHP_EOL . "Deny from all";
			$rules .= PHP_EOL . "</FilesMatch>";
			$rules .= PHP_EOL . "## End Securitycheck Pro Prevent access to .ht files" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que proteger los directorios de navegacin no autorizada */
	if ( $this->getValue("prevent_unauthorized_browsing") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['prevent_unauthorized_browsing'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro Prevent Unauthorized Browsing";
			$rules .= PHP_EOL . "Options All -Indexes";
			$rules .= PHP_EOL . "## End Securitycheck Pro Prevent Unauthorized Browsing" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que proteger frente a ataques de inclusin */
	if ( $this->getValue("file_injection_protection") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['file_injection_protection'] )) ) {
			$rules .= PHP_EOL . "## Begin File Injection Protection";
			$rules .= PHP_EOL . "RewriteCond %{REQUEST_METHOD} GET";
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} [a-zA-Z0-9_]=http:// [OR]";
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} [a-zA-Z0-9_]=(\.\.//?)+ [OR]";
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} [a-zA-Z0-9_]=/([a-z0-9_.]//?)+ [NC]";
			$rules .= PHP_EOL . "RewriteRule .* - [F]";
			$rules .= PHP_EOL . "## End File Injection Protection" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que proteger frente a ataques que intentan explotar la vulnerabilidad de /proc/self/environ */
	if ( $this->getValue("self_environ") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['file_injection_protection'] )) ) {
			$rules .= PHP_EOL . "## /proc/self/environ? Go away!";
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} proc/self/environ [NC,OR]" . PHP_EOL;			
		}
	}
	
	/* Comprobamos si hay que proteger las cabeceras X-Frame del navegador */
	if ( !($this->getValue("xframe_options") == 'NO') ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['xframe_options'] )) ) {
			$rules .= PHP_EOL . "## Don't allow any pages to be framed - Defends against CSRF";
			$rules .= PHP_EOL . "Header set X-Frame-Options " . $this->getValue("xframe_options") . PHP_EOL;			
		}
	}
	
	/* Comprobamos si hay que establecer proteccin contra ataques basados en 'mime'*/
	if ( $this->getValue("prevent_mime_attacks") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['prevent_mime_attacks'] )) ) {
			$rules .= PHP_EOL . "## Prevent mime based attacks";
			$rules .= PHP_EOL . 'Header set X-Content-Type-Options "nosniff"' . PHP_EOL;			
		}
	}
	
	/* Comprobamos si hay que aplicar la lista de user-agents por defecto */
	if ( $this->getValue("default_banned_list") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['default_banned_list'] )) ) {
			$user_agent_rules = JFile::read(JPATH_COMPONENT_ADMINISTRATOR.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'user_agent_blacklist.inc');
			// Aadimos el contenido del fichero por defecto al final del buffer
			$rules .= PHP_EOL . $user_agent_rules . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que aplicar la lista de user-agents establecida por el usuario */
	$own_user_agents = explode( PHP_EOL, $this->getValue("own_banned_list") );
	if ( ! empty( $own_user_agents ) && ! ( sizeof( $own_user_agents ) == 1 && trim( $own_user_agents[0] ) == ''  ) ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['own_banned_list'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro User Own Blacklist";
			$count = 1;
					
			foreach ( $own_user_agents as $agent ) {
					
				$rules .= PHP_EOL . "RewriteCond %{HTTP_USER_AGENT} " . trim( $agent );
								
				if ( $count < sizeof( $own_user_agents ) ) {
					$rules .= " [NC,OR]";
					$count++;
				} else {
					$rules .= " [NC]";
				}
			}
				
			$rules .= PHP_EOL . "RewriteRule ^(.*)$ - [F,L]";
			$rules .= PHP_EOL . "## End Securitycheck Pro User Own Blacklist" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que deshabilitar la firma del servidor*/
	if ( $this->getValue("disable_server_signature") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['disable_server_signature'] )) ) {
			$rules .= PHP_EOL . "# Begin Securitycheck Pro Disable Server Signature";
			$rules .= PHP_EOL . "ServerSignature Off";
			$rules .= PHP_EOL . "## End Securitycheck Pro Disable Server Signature" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que prohibir los 'easter-eggs' de PHP */
	if ( $this->getValue("disallow_php_eggs") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['disallow_php_eggs'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro Disallow Php Easter Eggs";
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} \=PHP[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12} [NC]";
			$rules .= PHP_EOL . "RewriteRule .* index.php [F]";
			$rules .= PHP_EOL . "## End Securitycheck Pro Disallow Php Easter Eggs" . PHP_EOL;
		}
	}
	
	/* Comprobamos si hay que prohibir el acceso a archivos que pueden contener informacin sensible */
	if ( $this->getValue("disallow_sensible_files_access") ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['disallow_sensible_files_access'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro Disallow Access To Sensitive Files";
			$rules .= PHP_EOL . "RewriteRule ^(htaccess.txt|configuration.php(-dist)?|joomla.xml|README.txt|web.config.txt|CONTRIBUTING.md|phpunit.xml.dist)$ - [F]";
			$rules .= PHP_EOL . "## End Securitycheck Pro Disallow Access To Sensitive Files";
		}
	}
	
	/* Comprobamos si hay que ocultar la url del backend */
	if ( !is_null($this->getValue("hide_backend_url")) ) {
		if ( ( !$ExistsHtaccess ) || (( $ExistsHtaccess ) &&  ( !$this->ConfigApplied['hide_backend_url'] )) ) {
			$rules .= PHP_EOL . "## Begin Securitycheck Pro Hide Backend Url";
			$rules .= PHP_EOL . "RewriteCond %{HTTP_REFERER} !" . $site_url;
			$rules .= PHP_EOL . "RewriteCond %{QUERY_STRING} !^" . $this->getValue("hide_backend_url");
			$rules .= PHP_EOL . "RewriteRule ^.*administrator/? /not_found [R,L]";
			$rules .= PHP_EOL . "## End Securitycheck Pro Hide Backend Url";
		}
	}
	
	/* Comprobamos si hay algo que aplicar */
	if ( !is_null($rules) ) {
		// Escribimos el contenido del buffer en el fichero '.htaccess'
		$status = JFile::write(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess', $rules);
	}
	
	return $status;
}

/* Borra el fichero .htaccess*/
function delete_htaccess()
{
	return JFile::delete(JPATH_SITE.DIRECTORY_SEPARATOR.'.htaccess');
}

/*Genera la clave secreta para la funcin de ocultar el backend */
function generateKey() {
	
	// Obtenemos la longitud de la clave que tenemos que generar
	$params = JComponentHelper::getParams('com_securitycheckpro');
	$size = $params->get('secret_key_length',20);

	$chars = "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"; //available characters
	srand( ( double ) microtime() * 1000000 ); //random seed
	$pass = '' ;
		
	for ( $i = 1; $i <= $size; $i++ ) {
		$num = rand() % 33;
		$tmp = substr( $chars, $num, 1 );
		$pass = $pass . $tmp;
	}

	return $pass;	
}

/*Genera las reglas equivalentes a .htaccess en ficheros NGINX */
function generate_rules() {

	$rules = null;
	
	/* Comprobamos si hay que proteger los archivos .ht */
	if ( $this->getValue("prevent_access") ) {
		$rules .= PHP_EOL . "# Begin Securitycheck Pro Prevent access to .ht files" . PHP_EOL;
		$rules .= "\tlocation ~ /\.ht { deny all; }" . PHP_EOL;
		$rules .= "# End Securitycheck Pro Prevent access to .ht files" . PHP_EOL;		
	}
	
	/* Comprobamos si hay que aplicar la lista de user-agents por defecto */
	if ( $this->getValue("default_banned_list") ) {
		$user_agent_rules = JFile::read(JPATH_COMPONENT_ADMINISTRATOR.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'user_agent_blacklist_nginx.inc');
		// Aadimos el contenido del fichero por defecto al final del buffer
		$rules .= PHP_EOL . $user_agent_rules . PHP_EOL;
	}
	
	/* Comprobamos si hay que aplicar la lista de user-agents establecida por el usuario */
	$own_user_agents = explode( PHP_EOL, $this->getValue("own_banned_list") );
	if ( ! empty( $own_user_agents ) && ! ( sizeof( $own_user_agents ) == 1 && trim( $own_user_agents[0] ) == ''  ) ) {
		$rules .= PHP_EOL . "# Begin Securitycheck Pro User Own Blacklist" . PHP_EOL;
		$count = 1;
		$nginx_list = '';
					
		foreach ( $own_user_agents as $agent ) {
				
			$nginx_list .= trim( $agent );
							
			if ( $count < sizeof( $own_user_agents ) ) {
				$nginx_list .= '|';
				$count++;
			} 
		}
				
		$rules .= "\tif (\$http_user_agent ~* " . $nginx_list . ") { return 403; }" . PHP_EOL;
		$rules .= "# End Securitycheck Pro User Own Blacklist" . PHP_EOL;
	}
	
	/* Comprobamos si hay que deshabilitar la firma del servidor*/
	if ( $this->getValue("disable_server_signature") ) {
			$rules .= PHP_EOL . "# Begin Securitycheck Pro Disable Server Signature" . PHP_EOL;
			$rules .= "server_tokens off;" . PHP_EOL;
			$rules .= "# End Securitycheck Pro Disable Server Signature" . PHP_EOL;
	}
	
	/* Comprobamos si hay que prohibir los 'easter-eggs' de PHP */
	if ( $this->getValue("disallow_php_eggs") ) {
		$rules .= PHP_EOL . "# Begin Securitycheck Pro Disallow Php Easter Eggs" . PHP_EOL;
		$rules .= "\tset \$susquery 0;" . PHP_EOL;
		$rules .= "\tif (\$args ~* \"=PHP[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}\") { set \$susquery 1; }" . PHP_EOL;
		$rules .= "\tif (\$susquery = 1) { return 403; }" . PHP_EOL;
		$rules .= "# End Securitycheck Pro Disallow Php Easter Eggs" . PHP_EOL;
	}
	
	/* Comprobamos si hay que prohibir el acceso a archivos que pueden contener informacin sensible */
	if ( $this->getValue("disallow_sensible_files_access") ) {
		$rules .= PHP_EOL . "# Begin Securitycheck Pro Disallow Access To Sensitive Files" . PHP_EOL;
		$rules .=  "\trewrite ^/(htaccess.txt|configuration.php(-dist)?|joomla.xml|README.txt|web.config.txt|CONTRIBUTING.md|phpunit.xml.dist)$ /not_found last;" . PHP_EOL;
		$rules .= "# End Securitycheck Pro Disallow Access To Sensitive Files" . PHP_EOL;
	}
	
	/* Comprobamos si hay que ocultar la url del backend */
	if ( !is_null($this->getValue("hide_backend_url")) ) {
		$rules .= "# Begin Securitycheck Pro Hide Backend Url" . PHP_EOL;
		$rules .= "\tset \$rule_1 0;" . PHP_EOL;
		$rules .= "\tif (\$http_referer !~* administrator ) { set \$rule_1 6\$rule_1; }" . PHP_EOL;
		$rules .= "\tif (\$args !~ \"^" . $this->getValue("hide_backend_url") . "\") { set \$rule_1 9\$rule_1; }" . PHP_EOL;
		$rules .= "\tif (\$rule_1 = 960) {" . PHP_EOL;
		$rules .= "\t\trewrite ^(.*/)?administrator /not_found redirect;" . PHP_EOL;
		$rules .= "\t\trewrite ^/administrator(.*)$ /not_found redirect;" . PHP_EOL;
		$rules .= "\t}" . PHP_EOL;
		$rules .= "# End Securitycheck Pro Hide Backend Url";
	}
	
	return $rules;
 
}

}