<?php
/**
 * Securitycheck Pro package
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');
jimport('joomla.installer.installer');

/**
 * Script file of Securitycheck Pro component
 */
class com_SecuritycheckproInstallerScript {
	// Check if we are calling update method. It's used in 'install_message' function
	public $update = false;
	
	// Resultado de la desinstalacin del componente Securitycheck
	public $result_free = "";
	public $id_free;
	
	/** @var array Obsolete files and folders to remove  */
	private $ObsoleteFilesAndFolders = array(
		'files'	=> array(
			// Outdated media files
			'media/com_securitycheckpro/images/blocked.jpg',
			'media/com_securitycheckpro/images/dinamically_blocked.jpg',
			'media/com_securitycheckpro/images/http.jpg',
			'media/com_securitycheckpro/images/no_read.jpg',
			'media/com_securitycheckpro/images/oval_blue_left.gif',
			'media/com_securitycheckpro/images/oval_blue_right.gif',
			'media/com_securitycheckpro/images/oval_green_left.gif',
			'media/com_securitycheckpro/images/oval_green_right.gif',
			'media/com_securitycheckpro/images/permitted.jpg',
			'media/com_securitycheckpro/images/read.jpg',
			'media/com_securitycheckpro/images/second_level.jpg',
			'media/com_securitycheckpro/images/session_hijack.jpg',
			'media/com_securitycheckpro/images/session_protection.jpg',
			'media/com_securitycheckpro/images/task_running.gif',
			
		)
	);
	
	/* Hace una consulta a la tabla #__securitycheckpro_storage para consultar si se han migrado los parmetros del plugin Firewall Web  */
	private function check_plugin_migration($plugin_name)
	{
		$db = JFactory::getDBO();
		$query = $db->getQuery(true);
		$query 
			->select($db->quoteName('storage_value'))
			->from($db->quoteName('#__securitycheckpro_storage'))
			->where($db->quoteName('storage_key').' = '.$db->quote($plugin_name));
		$db->setQuery($query);
		$res = $db->loadResult();
			
		if(!empty($res)) {
			return true;
		} else {
			return false;
		}
	}
	
	/* Migramos los valores del plugin Web Firewall a la tabla '#__securitycheckpro_storage' */
	private function migrate_plugin_values($plugin_name) {
		//Inicializamos las variables
		$clave = null;
		
		if ( $plugin_name == 'pro_plugin' ) {
			$clave = 'System - Securitycheck Pro';
		} else if ( $plugin_name == 'cron_plugin' ) {
			$clave = 'System - Securitycheck Pro Cron';
		}
				
		// Obtenemos los valores de las distintas opciones del Firewall Web
		$db = JFactory::getDBO();
		$query = $db->getQuery(true)
			->select(array($db->quoteName('params')))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('name').' = '.$db->quote($clave));
		$db->setQuery($query);
		$params = $db->loadResult();
		
		// Los replicamos en la nueva tabla
		$object = (object)array(
			'storage_key'		=> $plugin_name,
			'storage_value'		=> $params
		);
		
		$migration_result = $db->insertObject('#__securitycheckpro_storage', $object);
		return $migration_result;
	
	}
	
	/* Funcin que desinstala el componente Securitycheck */
	private function _unistall_Securitycheck() {
		
		$db = JFactory::getDbo();
		$installer = new JInstaller();
		
		$columnName      = $db->quoteName("extension_id");
		$tableExtensions = $db->quoteName("#__extensions");
		$type 			 = $db->quoteName("type");
		$columnElement   = $db->quoteName("element");

		// Uninstall Securitycheck component
		$db->setQuery(
				"SELECT 
					$columnName
				FROM
					$tableExtensions
				WHERE
					$type='component'
				AND
					$columnElement='com_securitycheck'"		
		);

		$this->id_free = $db->loadResult();

		if ($this->id_free) {
			$this->result_free = $installer->uninstall('component',$this->id_free,1);
		}
	}
	
	/**
	 * Removes obsolete files and folders
	 *
	 * @param array $ObsoleteFilesAndFolders
	 */
	private function _removeObsoleteFilesAndFolders($ObsoleteFilesAndFolders)
	{
		$securitycheckpro_cached_file = JPATH_CACHE . '/com_securitycheckpro.updates.ini';
		
		// Remove cached files
		if ( JFile::exists($securitycheckpro_cached_file) ) {
			JFile::delete($securitycheckpro_cached_file);
		}
		
		// Remove files
		JLoader::import('joomla.filesystem.file');
		if(!empty($ObsoleteFilesAndFolders['files'])) foreach($ObsoleteFilesAndFolders['files'] as $file) {
			$f = JPATH_ROOT.'/'.$file;
			if(!JFile::exists($f)) continue;
			JFile::delete($f);
		}
		
		/* Remove folders (Not used now, but could be useful in a future)
		JLoader::import('joomla.filesystem.file');
		if(!empty($ObsoleteFiles['folders'])) foreach($ObsoleteFiles['folders'] as $folder) {
			$f = JPATH_ROOT.'/'.$folder;
			if(!JFolder::exists($f)) continue;
			JFolder::delete($f);
		}*/
	}
	
	/**
	 * Joomla! pre-flight event
	 * 
	 * @param string $type Installation type (install, update, discover_install)
	 * @param JInstaller $parent Parent object
	 */
	public function preflight($type, $parent)
	{
		// Only allow to install on PHP 5.3.0 or later
		if ( !version_compare(PHP_VERSION, '5.3.0', 'ge') ) {
			Jerror::raiseWarning(null, "Securitycheck Pro requires, at least, PHP 5.3.0");
			return false;
		} else if ( version_compare(JVERSION, '3.0.0', 'lt') ) {
			// Only allow to install on Joomla! 3.0.0 or later, but not in 2.5 branch
			Jerror::raiseWarning(null, "This version doesn't work in Joomla! 2.5 branch");
			return false;
		}
		
		$this->_removeObsoleteFilesAndFolders($this->ObsoleteFilesAndFolders);
		
		$this->_unistall_Securitycheck();
	}
	
	/**
	 * Runs after install, update or discover_update
	 * @param string $type install, update or discover_update
	 * @param JInstaller $parent 
	 */
	function postflight( $type, $parent )
	{
		// Inicializamos las variables
		$existe_tabla = false;
		$web_firewall_migrado = false;
		$cron_migrado = false;
		
		$db = JFactory::getDBO();
		$total_rows = $db->getTableList();
		
		if ( !(is_null($total_rows)) ) {
			foreach ($total_rows as $table_name) {
				if ( strstr($table_name,"securitycheckpro_logs") ) {
					$existe_tabla = true;
				}
			}
		}
		
		if ( !$existe_tabla ) {
			// Disable Securitycheck Pro plugin
			$tableExtensions = $db->quoteName("#__extensions");
			$columnElement   = $db->quoteName("element");
			$columnType      = $db->quoteName("type");
			$columnEnabled   = $db->quoteName("enabled");
			$db->setQuery(
				"UPDATE 
					$tableExtensions
				SET
					$columnEnabled=0
				WHERE
					$columnElement='securitycheckpro'
				AND
					$columnType='plugin'"
			);
			$db->query();
			
			// Disable Securitycheck Pro Cron plugin
			$db->setQuery(
				"UPDATE 
					$tableExtensions
				SET
					$columnEnabled=0
				WHERE
					$columnElement='securitycheckpro_cron'
				AND
					$columnType='plugin'"
			);

		$db->query();
			Jerror::raiseWarning(null, "There has been an error when creating database tables. Securitycheck Pro Web Firewall and Cron plugin has been disabled.");
		}
		
		$web_firewall_migrado = $this->check_plugin_migration('pro_plugin');
		if ( !$web_firewall_migrado ) {
			$migration_result = $this->migrate_plugin_values('pro_plugin');
			if ( $migration_result ) {
				JFactory::getApplication()->enqueueMessage("Web Firewall plugin values has been migrated.");		
			} else {
				Jerror::raiseWarning(null, JText::sprintf("There has been an error migrating Web Firewall plugin."));
			}
		}
		
		$cron_migrado = $this->check_plugin_migration('cron_plugin');
		if ( !$cron_migrado ) {
			$migration_result = $this->migrate_plugin_values('cron_plugin');
			if ( $migration_result ) {
				JFactory::getApplication()->enqueueMessage("Cron plugin values has been migrated.");		
			} else {
				Jerror::raiseWarning(null, JText::sprintf("There has been an error migrating Cron plugin."));
			}
		}
		
	}
	
	/**
	 * method to install the component
	 *
	 * @return void
	 */
	function install($parent) {
		// General settings
		$status = new JObject();
		$status->modules = array();
		
		// Array to store module and plugin installation results
		$result = array();
		$indice = 0;
		
		$manifest = $parent->get("manifest");
		$parent = $parent->getParent();
		$source = $parent->getPath("source");

		$installer = new JInstaller();
					
		// Install module
		$db = JFactory::getDbo();
		$result[$indice] = $installer->install($source. DIRECTORY_SEPARATOR .'modules' . DIRECTORY_SEPARATOR .'mod_scpadmin_quickicons');
		$indice++;
				
		// Enable and configure module
		$query = "UPDATE #__modules SET position='icon', ordering = '-10', published = '1' WHERE `module`='mod_scpadmin_quickicons'";
		$db->setQuery($query);
		$db->query();

		$query = "SELECT `id` FROM `#__modules` WHERE `module` = 'mod_scpadmin_quickicons'";
		$db->setQuery($query);
		(int) $modID = $db->loadResult();
		
		$query = "REPLACE INTO `#__modules_menu` (`moduleid`,`menuid`) VALUES ({$modID}, 0)";
		$db->setQuery($query);
		$db->query();
				
		$status->modules[] = array('name'=>'Securitycheck Pro - Quick Icons','client'=>'administrator', 'result'=>$result); 
		
		// Install plugins
						
		foreach($manifest->plugins->plugin as $plugin) {
			$attributes = $plugin->attributes();
			$plg = $source . DIRECTORY_SEPARATOR . $attributes['folder'].DIRECTORY_SEPARATOR.$attributes['plugin'];
			$result[$indice] = $installer->install($plg);
			$indice++;
		}

		$db = JFactory::getDbo();
		$tableExtensions = $db->quoteName("#__extensions");
		$columnElement   = $db->quoteName("element");
		$columnType      = $db->quoteName("type");
		$columnEnabled   = $db->quoteName("enabled");
            
		// Enable Securitycheck Pro plugin
		$db->setQuery(
			"UPDATE 
				$tableExtensions
			SET
				$columnEnabled=1
			WHERE
				$columnElement='securitycheckpro'
			AND
				$columnType='plugin'"
		);

		$db->query();
		
		// Enable Securitycheck Pro Cron plugin
		$db->setQuery(
			"UPDATE 
				$tableExtensions
			SET
				$columnEnabled=1
			WHERE
				$columnElement='securitycheckpro_cron'
			AND
				$columnType='plugin'"
		);

		$db->query();
		
		// Install message
		$this->install_message($this->id_free,$this->result_free,$result,$status); 
	}
	
	/**
	 * method to uninstall the component
	 *
	 * @return void
	 */
	function uninstall($parent){
	
		// General settings
		$status = new JObject();
		$status->modules = array();
		
		// Array to store uninstall results
		$result = array();
		
		$db = JFactory::getDbo();
		
		// Uninstall module
		$db->setQuery("SELECT extension_id FROM #__extensions WHERE type = 'module' AND element = 'mod_scpadmin_quickicons' LIMIT 1");
		(int) $id = $db->loadResult();
		if ($id) {
			$installer = new JInstaller();
			$result[0] = $installer->uninstall('module', $id);
			$status->modules[] = array('name'=>'Securitycheck Pro - Quick Icons','client'=>'administrator', 'result'=>$result);			
		}
		
		$columnName      = $db->quoteName("extension_id");
		$tableExtensions = $db->quoteName("#__extensions");
		$type 			 = $db->quoteName("type");
		$columnElement   = $db->quoteName("element");
		$columnType      = $db->quoteName("folder");
		$result = '';
            
		// Uninstall  Securitycheck Pro plugin
		$db->setQuery(
			"SELECT 
				$columnName
			FROM
				$tableExtensions
			WHERE
				$type='plugin'
			AND
				$columnElement='securitycheckpro'
			AND
				$columnType='system'"
		
		);

		$id = $db->loadResult();

		if ($id) {
			$installer = new JInstaller();
			$result[1] = $installer->uninstall('plugin',$id,1);		
		}
		
		// Uninstall  Securitycheck Pro Cron plugin
		$db->setQuery(
			"SELECT 
				$columnName
			FROM
				$tableExtensions
			WHERE
				$type='plugin'
			AND
				$columnElement='securitycheckpro_cron'
			AND
				$columnType='system'"
		
		);

		$id = $db->loadResult();

		if ($id) {
			$installer = new JInstaller();
			$result[2] = $installer->uninstall('plugin',$id,1);		
		}
		
		// Uninstall message
		$this->uninstall_message($result,$status);
		
	}
				
	/**
	 * method to update the component
	 *
	 * @return void
	 */
	function update($parent) {
		// This variable is updated.
		$this->update = true;
		$this->install($parent);		
	}
	
	/**
	 * method to show the install message
	 *
	 * @return void
	 */
	function install_message($id_free,$result_free,$result,$status){
		// Initialize variables
		$cabecera = '';
		$result_ok = '';
		$result_not_ok = '';
			
		if ( !($this->update) ) {
			$cabecera = JText::_( 'COM_SECURITYCHECKPRO_HEADER_INSTALL' );
			$result_ok = JText::_( 'COM_SECURITYCHECKPRO_INSTALLED' );
			$result_not_ok = JText::_( 'COM_SECURITYCHECKPRO_NOT_INSTALLED' );
		} else {
			$cabecera = JText::_( 'COM_SECURITYCHECKPRO_HEADER_UPDATE' );
			$result_ok = JText::_( 'COM_SECURITYCHECKPRO_UPDATED' );
			$result_not_ok = JText::_( 'COM_SECURITYCHECKPRO_NOT_UPDATED' );
		}
?>
		<img src='../media/com_securitycheckpro/images/tick_48x48.png' style='float: left; margin: 5px;'>
		<?php
			if ( !($this->update) ) {			
		?>
			<h1><?php echo $cabecera ?></h1>
			<h2><?php echo JText::_( 'COM_SECURITYCHECKPRO_WELCOME' ); ?></h2>
		<?php 
			} else {
		?>
			<h2><?php echo $cabecera ?></h2>
		<?php
			}
		?>
			<div class="securitycheck-bootstrap">
			<table class="table table-striped">
				<thead>
					<tr>
						<th class="title" colspan="2"><?php echo JText::_( 'COM_SECURITYCHECKPRO_EXTENSION' ); ?></th>
						<th width="30%"><?php echo JText::_( 'COM_SECURITYCHECKPRO_STATUS' ); ?></th>
					</tr>
				</thead>
				<tfoot>
					<tr>
						<td colspan="3"></td>
					</tr>
				</tfoot>
				<tbody>
					<tr>
						<td colspan="2">Securitycheck Pro <?php echo JText::_( 'COM_SECURITYCHECKPRO_COMPONENT' ); ?></td>
						<td><strong><?php echo $result_ok ?></strong>
						<img src="../media/com_securitycheckpro/images/ok_small.png" title="<?php echo $result_ok ?>" alt="<?php echo $result_ok ?>"></td>
					</tr>
					<tr class="row0">
						<td class="key" colspan="2">Securitycheck Pro <?php echo JText::_( 'COM_SECURITYCHECKPRO_PLUGIN' ); ?></td>
					<?php
						if ( $result[1] ) {
					?>
							<td><strong><?php echo $result_ok ?></strong>
							<img src="../media/com_securitycheckpro/images/ok_small.png" title="<?php echo $result_ok ?>" alt="<?php echo $result_ok ?>"></td>
					<?php
						} else {
					?>
							<td><strong><?php echo $result_not_ok ?></strong>
							<img src="../media/com_securitycheckpro/images/error_small.png" title="<?php echo $result_not_ok ?>" alt="<?php echo $result_not_ok ?>"></td>
					<?php
						}
					?>
					</tr>
					<tr class="row0">
						<td class="key" colspan="2">Securitycheck Pro Cron <?php echo JText::_( 'Plugin' ); ?></td>
					<?php
						if ( $result[2] ) {
					?>
						<td><strong><?php echo $result_ok ?></strong>
							<img src="../media/com_securitycheckpro/images/ok_small.png" title="<?php echo $result_ok ?>" alt="<?php echo $result_ok ?>"></td>
					<?php
						} else {
					?>
						<td><strong><?php echo $result_not_ok ?></strong>
							<img src="../media/com_securitycheckpro/images/error_small.png" title="<?php echo $result_not_ok ?>" alt="<?php echo $result_not_ok ?>"></td>
					<?php
						}
					?>
					</tr>
					<?php
					if ( count($status->modules) > 0 ) {
					?>
						<tr class="row0">
						<td class="key" colspan="2">Securitycheck Pro Info <?php echo JText::_( 'COM_SECURITYCHECKPRO_MODULE' ); ?></td>
					<?php
							if ($status->modules['0']['result']) {
					?>
							<td><strong><?php echo $result_ok ?></strong>
								<img src="../media/com_securitycheckpro/images/ok_small.png" title="<?php echo $result_ok ?>" alt="<?php echo $result_ok ?>"></td>
					<?php
							} else {
					?>
								<td><strong><?php echo $result_not_ok ?></strong>
								<img src="../media/com_securitycheckpro/images/error_small.png" title="<?php  echo $result_not_ok ?>" alt="<?php echo $result_not_ok ?>"></td>
					<?php
							}
					?>
						</tr>
					<?php
					}
					if ($id_free) {
					?>
						<tr class="row0">
							<td class="key" colspan="2">Securitycheck <?php echo JText::_( 'COM_SECURITYCHECK_COMPONENT' ); ?></td>
					<?php
							if ($result_free) {
					?>
								<td><strong><?php echo JText::_( 'COM_SECURITYCHECK_UNINSTALLED' ); ?></strong>
								<img src="../media/com_securitycheckpro/images/ok_small.png" title="<?php echo JText::_( 'COM_SECURITYCHECK_UNINSTALLED '); ?>" alt="<?php echo JText::_( 'COM_SECURITYCHECK_UNINSTALLED' ); ?>"></td>
					<?php
							} else {
					?>
								<td><strong><?php echo JText::_( 'COM_SECURITYCHECK_NOT_UNINSTALLED' ); ?></strong>
								<img src="../media/com_securitycheckpro/images/error_small.png" title="<?php echo JText::_( 'COM_SECURITYCHECK_NOT_UNINSTALLED' ); ?>" alt="<?php echo JText::_( 'COM_SECURITYCHECK_NOT_UNINSTALLED' ); ?>"></td>
					<?php
							}
					?>
						</tr>
					<?php
					}
					?>
				</tbody>
			</table>
			</div>
<?php
	}

	/**
	 * method to show the uninstall message
	 *
	 * @return void
	 */
	function uninstall_message($result,$status){
?>
		<h1><?php echo JText::_( 'COM_SECURITYCHECKPRO_HEADER_UNINSTALL' ); ?></h1>
		<h2><?php echo JText::_( 'COM_SECURITYCHECKPRO_GOODBYE' ); ?></h2>
		<div class="securitycheck-bootstrap">
		<table class="table table-striped">
			<thead>
				<tr>
					<th class="title" colspan="2"><?php echo JText::_( 'COM_SECURITYCHECKPRO_EXTENSION' ); ?></th>
					<th width="30%"><?php echo JText::_( 'COM_SECURITYCHECKPRO_STATUS' ); ?></th>
				</tr>
			</thead>
			<tfoot>
				<tr>
					<td colspan="3"></td>
				</tr>
			</tfoot>
			<tbody>
				<tr>
					<td colspan="2">Securitycheck Pro <?php echo JText::_( 'COM_SECURITYCHECKPRO_COMPONENT' ); ?></td>
					<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_UNINSTALLED' ); ?></strong>
				</tr>
				<tr class="row0">
					<td class="key" colspan="2">Securitycheck Pro <?php echo JText::_( 'COM_SECURITYCHECKPRO_PLUGIN' ); ?></td>
				<?php
					if ($result[1]) {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_UNINSTALLED' ); ?></strong>
				<?php
					} else {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_NOT_INSTALLED' ); ?></strong>
				<?php
					}
				?>
				</tr>
				<tr class="row0">
					<td class="key" colspan="2">Securitycheck Pro Cron <?php echo JText::_( 'Plugin' ); ?></td>
				<?php
					if ($result[2]) {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_UNINSTALLED' ); ?></strong>
				<?php
					} else {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_NOT_INSTALLED' ); ?></strong>
				<?php
					}
				?>
				</tr>
				<?php
				if ( count($status->modules) > 0 ) {
				?>
					<tr class="row0">
					<td class="key" colspan="2">Securitycheck Pro Info <?php echo JText::_( 'COM_SECURITYCHECKPRO_MODULE' ); ?></td>
				<?php
					if ($status->modules['0']['result']) {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_UNINSTALLED' ); ?></strong>
				<?php
					} else {
				?>
						<td><strong><?php echo JText::_( 'COM_SECURITYCHECKPRO_NOT_INSTALLED' ); ?></strong>
				<?php
					}
				?>
					</tr>
				<?php
				}
				?>
			</tbody>
		</table>
		</div>
<?php
	}
}
?>