<?php
/**
* Modelo MalwarescanStatus para el Componente Securitycheckpro
* @ author Jose A. Luque
* @ Copyright (c) 2011 - Jose A. Luque
* @license GNU/GPL v2 or later http://www.gnu.org/licenses/gpl-2.0.html
*/

// Chequeamos si el archivo est includo en Joomla!
defined('_JEXEC') or die();
jimport( 'joomla.application.component.model' );
jimport( 'joomla.version' );
jimport( 'joomla.access.rule' );
jimport( 'joomla.application.component.helper' );
jimport('joomla.updater.update' );
jimport('joomla.installer.helper' );
jimport('joomla.installer.installer' );
jimport( 'joomla.application.component.controller' );

/**
* Modelo FilesIntegrity
*/
class SecuritycheckprosModelMalwarescanStatus extends SecuritycheckproModel
{
/** @var object Pagination */
var $_pagination = null;

/** @var int Total number of files of Pagination */
var $total = 0;

/** @var array The files to process */
private $Stack_Malwarescan = array();

/** @var int Total numbers of file/folders in this site */
public $files_scanned_malwarescan = 0;

/** @var int Numbers of suspicious files */
public $suspicious_files = 0;

/** @var date Last malware scan check */
public $last_check_malwarescan = null;

/** @var string Path to the folder where scans will be stored */
private $folder_path = '';

/** @var string fileintegrity's name */
private $malwarescan_name = '';

/** @var string Use the same file integrity excepcions */
private $use_filemanager_exceptions = 1;

/* Number of analyzed files during the last hour */
private $analized_files_last_hour = 0;

/* Array with path keys analyzed */
private $analized_keys_array = array();

/* Number of hashes analyzed during the last hour */
private $analized_hashes_last_hour = 0;

function __construct()
{
	parent::__construct();
	
	// Establecemos la ruta donde se almacenarn los escaneos
	$this->folder_path = JPATH_ADMINISTRATOR.DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_securitycheckpro'.DIRECTORY_SEPARATOR.'scans'.DIRECTORY_SEPARATOR;
	
	// Obtenemos el nombre de los escaneos anteriores
	$db = $this->getDbo();
	$query = $db->getQuery(true)
		->select(array($db->quoteName('storage_value')))
		->from($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote('malwarescan_resume'));
	$db->setQuery($query);
	$stack_malwarescan = $db->loadResult();	
	$stack_malwarescan = json_decode($stack_malwarescan, true);
	
	if(!empty($stack_malwarescan)) {
		$this->malwarescan_name = $stack_malwarescan['filename'];
	}
	
	// Establecemos el tamao mximo de memoria que el script puede consumir
	$params = JComponentHelper::getParams('com_securitycheckpro');
	$memory_limit = $params->get('memory_limit','512M');
	ini_set('memory_limit',$memory_limit);

	$mainframe = JFactory::getApplication();
 
	// Obtenemos las variables de paginacin de la peticin
	$limit = $mainframe->getUserStateFromRequest('global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int');
	$limitstart = JRequest::getVar('limitstart', 0, '', 'int');

	// En el caso de que los lmites hayan cambiado, los volvemos a ajustar
	$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);
	
	/* Limitamos a 100 el nmero de archivos mostrados para evitar que el array desborde la memoria mxima establecida por PHP */
	if ( $limit == 0 ){
		$this->setState('limit', 100);
		$this->setState('showall', 1);
	} else {
		$this->setState('limit', $limit);
	}
	$this->setState('limitstart', $limitstart);
			
	// Chequeamos si ha pasado ms de una hora desde el ltimo escaneo online para inicializar la variable que la controla
	$this->check_last_onlinecheck();
}


/* Funcin que obtiene un array con los datos que sern mostrados en la opcin 'malwarescan' */
function loadStack($opcion,$field)
{
	(int) $lower_limit = 0; 
	
	// Establecemos el tamao mximo de memoria que el script puede consumir
	$params = JComponentHelper::getParams('com_securitycheckpro');
	$memory_limit = $params->get('memory_limit','512M');
	ini_set('memory_limit',$memory_limit);
	
	$db = $this->getDbo();
	
	switch ($opcion) {
		case "malwarescan":
			// Leemos el contenido del fichero
			if ( JFile::exists($this->folder_path.DIRECTORY_SEPARATOR.$this->malwarescan_name) ) {
				$stack = JFile::read($this->folder_path.DIRECTORY_SEPARATOR.$this->malwarescan_name);
				// Eliminamos la parte del fichero que evita su lectura al acceder directamente
				$stack = str_replace("#<?php die('Forbidden.'); ?>",'',$stack);
			}
			
			if(empty($stack)) {
				$this->Stack_Malwarescan = array();
				return;
			}
			break;
		case "malwarescan_resume":
			$query = $db->getQuery(true)
				->select(array($db->quoteName('storage_value')))
				->from($db->quoteName('#__securitycheckpro_storage'))
				->where($db->quoteName('storage_key').' = '.$db->quote('malwarescan_resume'));
			$db->setQuery($query);
			$stack = $db->loadResult();
			
			if(empty($stack)) {
				$this->files_scanned_malwarescan = 0;
				$this->suspicious_files = 0;
				return;
			}
			break;
	
	}
	
	$stack = json_decode($stack, true);
	
	/* Obtenemos el nmero de registros del array que hemos de mostrar. Si el lmite superior es '0', entonces devolvemos todo el array */
	$upper_limit = $this->getState('limitstart');
	$lower_limit = $this->getState('limit');
	
	/* Obtenemos los valores de los filtros */
	$filter_malwarescan_status = $this->state->get('filter.malwarescan_status');
	$search = htmlentities($this->state->get('filter.malwarescan_search'));
		
	switch ($field) {
		case "malwarescan":
			$filtered_array = array();
			/* Si el campo 'search' no est vaco, buscamos en todos los campos del array */			
			if (!empty($search) ) {
				$filtered_array = array_values(array_filter($stack['files_folders'], function ($element) use ($filter_malwarescan_status,$search) { return ( ($element['safe_malwarescan'] == $filter_malwarescan_status) && ( (strstr($element['path'],$search)) || (strstr($element['size'],$search)) || (strstr($element['last_modified'],$search)) || (strstr($element['malware_type'],$search)) || (strstr($element['malware_description'],$search)) ) );} ));
			} else {
				$filtered_array = array_values(array_filter($stack['files_folders'], function ($element) use ($filter_malwarescan_status) { return ( ($element['safe_malwarescan'] == $filter_malwarescan_status) );} ));
			}
			$this->total = count($filtered_array);
			/* Cortamos el array para mostrar slo los valores mostrados por la paginacin */
			$this->Stack_Malwarescan = array_splice($filtered_array, $upper_limit, $lower_limit);
			return ($this->Stack_Malwarescan);
		case "files_scanned_malwarescan":
			$this->files_scanned_malwarescan = $stack['files_scanned_malwarescan'];
			return ($this->files_scanned_malwarescan);
		case "suspicious_files":
			$this->suspicious_files = $stack['suspicious_files'];
			return ($this->suspicious_files);
	}

}

protected function populateState()
{
	// Inicializamos las variables
	$app		= JFactory::getApplication();

	$malwarescan_search = $app->getUserStateFromRequest('filter.malwarescan_search', 'filter_malwarescan_search');
	$this->setState('filter.malwarescan_search', $malwarescan_search);
	$malwarescan_status = $app->getUserStateFromRequest('filter.malwarescan_status', 'filter_malwarescan_status');
	$this->setState('filter.malwarescan_status', $malwarescan_status);
	
						
	parent::populateState();
}

/*  Funcin para la paginacin */
function getPagination()
{
// Cargamos el contenido si es que no existe todava
if (empty($this->_pagination)) {
	jimport('joomla.html.pagination');
$this->_pagination = new JPagination($this->total, $this->getState('limitstart'), $this->getState('limit') );
}
return $this->_pagination;
}

/* Funcin para establecer el valor de un campo de la tabla '#_securitycheckpro_file_manager' */
function set_campo_filemanager($campo,$valor)
{
	// Creamos el nuevo objeto query
	$db = $this->getDbo();
	$query = $db->getQuery(true);
	
	// Sanitizamos las entradas
	$campo_sanitizado = $db->escape($campo);
	$valor_sanitizado = $db->Quote($db->escape($valor));

	// Construimos la consulta...
	$query->update('#__securitycheckpro_file_manager');
	$query->set($campo_sanitizado .'=' .$valor_sanitizado);
	$query->where('id=1');

	// ... y la lanzamos
	$db->setQuery( $query );
	$db->query();
}

/* Funcin para obtener el valor de un campo de la tabla '#_securitycheckpro_file_manager' */
function get_campo_filemanager($campo)
{
	try {
		// Creamos el nuevo objeto query
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		// Sanitizamos las entradas
		$campo_sanitizado = $db->Quote($db->escape($campo));
		
		// Construimos la consulta...
		$query->select($campo);
		$query->from('#__securitycheckpro_file_manager');
		$query->where('id=1');
		
		// ... y la lanzamos
		$db->setQuery( $query );
		$result = $db->loadResult();
	} catch (Exception $e) {
		$result = "ERROR";
	}
	
	if ( (is_null($result)) && ($campo=='estado') ) {
		$result = "ERROR";
	}
	
	// Devolvemos el resultado
	return $result;	
}

/* Funcin que guarda en la BBDD, en formato json, el contenido de un array con todos los ficheros y directorios */
private function saveStack(){
	// Creamos el nuevo objeto query
	$db = $this->getDbo();
	
	// Borramos el fichero del escaneo anterior...
	$delete_malwarescan_file = JFile::delete($this->folder_path.$this->malwarescan_name);
	
	try {
		$content_malwarescan = utf8_encode(json_encode(array('files_folders'	=> $this->Stack_Malwarescan)));
		$content_malwarescan = "#<?php die('Forbidden.'); ?>" . PHP_EOL . $content_malwarescan;
		$result_malwarescan = JFile::write($this->folder_path.$this->malwarescan_name, $content_malwarescan);
	} catch (Exception $e) {
		
	}	
	// Borramos el contenido previo de la BBDD
	$query = $db->getQuery(true)
		->delete($db->quoteName('#__securitycheckpro_storage'))
		->where($db->quoteName('storage_key').' = '.$db->quote('malwarescan_resume'));
	$db->setQuery($query);
	$db->query();
		
	$object = (object)array(
		'storage_key'	=> 'malwarescan_resume',
		'storage_value'	=> json_encode(array(
			'files_scanned_malwarescan'		=> $this->files_scanned_malwarescan,
			'suspicious_files'	=> $this->suspicious_files,
			'last_check_malwarescan'	=> $this->last_check_malwarescan,
			'filename'		=>$this->malwarescan_name
		))
	);
	$db->insertObject('#__securitycheckpro_storage', $object);
}
/* Chequea archivos contra el servicio OPWAST Metascan online */
function online_check_files()
{
	// Inicializamos las variables
	$this->analized_keys_array = array();
	$error = false;
	
	// Config.
	$api    = 'https://scan.metascan-online.com/v2/file';
	
	// Obtenemos la API key
	$params = JComponentHelper::getParams('com_securitycheckpro');
	$apikey = $params->get('opswat_key','');
	
	// Creamos el objeto JInput para obtener las variables del formulario
	$jinput = JFactory::getApplication()->input;
	
	// Obtenemos las rutas de los ficheros a analizar
	$paths = $jinput->get('malwarescan_status_table',null,'array');
			
	if ( !empty($paths) ) {	
		
		// Creamos el nuevo objeto query
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		// Cargamos el contenido del fichero de archivos sospechosos
		$malwarescan_data = $this->loadStack("malwarescan","malwarescan");
		// Creamos un array de rutas para modificar los elementos que hayan sido escaneados
		$array_paths = array_map( function ($element) { return $element['path']; },$malwarescan_data );
		
		// Nmero de archivos escaneados en la ltima hora
		$this->analized_files_last_hour = $this->get_online_analyzed_values("files");
		
		// Chequeamos si sobrepasamos el lmite de archivos a analizar por hora (25)
		if ( ($this->analized_files_last_hour) + (count($paths)) <= 25 ) {
			foreach($paths as $path) {		
				// Buscamos la clave del array a modificar
				$array_key = array_search($path,$array_paths);
				
				// Si tenemos un 'data_id' vlido no volvemos a preguntar por uno al servicio online. Esto significa que ya hemos remitido el fichero para su analisis.
				if ( empty($malwarescan_data[$array_key]['data_id']) ) {
							
					// Path sanitizada
					$file = $db->escape($path);
					
					// Build headers array.
					$headers = array(
						'apikey: '.$apikey,
						'filename: '.$file
					);

					// Build options array.
					$options = array(
						CURLOPT_URL     => $api,
						CURLOPT_HTTPHEADER  => $headers,
						CURLOPT_POST        => true,
						CURLOPT_POSTFIELDS  => file_get_contents($file),
						CURLOPT_RETURNTRANSFER  => true,
						CURLOPT_SSL_VERIFYPEER  => false
					);

					// Init & execute API call.
					$ch = curl_init();
					curl_setopt_array($ch, $options);
					$response = json_decode(curl_exec($ch), true);
					
					// Obtenemos el resultado de la consulta. Cualquier cdigo devuelto diferente a 200 indicar un error.
					$http_status = curl_getinfo($ch,CURLINFO_HTTP_CODE);
					
					if ( $http_status == 200 ) {
									
						// Buscamos la clave del array a modificar
						$array_key = array_search($path,$array_paths);
						// Almacenamos el valor encontrado para utilizarlo posteriormente
						array_push($this->analized_keys_array,$array_key);
						
						// Y aadimos los campos 'data_id' y 'rest_ip'
						$data_id = $response['data_id'];
						$rest_ip = $response['rest_ip'];
						$malwarescan_data[$array_key]['data_id'] = $data_id;
						$malwarescan_data[$array_key]['rest_ip'] = $rest_ip;
						
						// Incrementamos el valor de la variable de archivos analizados
						$this->analized_files_last_hour++;	
						
					} else {
						JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_ERROR_RETURNED',$http_status),'error');
					}
				} else {
					// Almacenamos el valor encontrado para utilizarlo posteriormente
					array_push($this->analized_keys_array,$array_key);					
				}
			}
			
			// Actualizamos los valores de los campos relacionados con el analisis online
			$this->set_campo_filemanager('online_checked_files',$this->analized_files_last_hour);
			$this->set_campo_filemanager('last_online_check_malwarescan',date('Y-m-d H:i:s'));		

			// Buscamos el resultado de los anlisis. Para ello preguntamos al servicio Metascan online sobre cada 'result_id' devuelto.
			$this->look_for_results($apikey,$malwarescan_data,"files");	
		} else {
			JFactory::getApplication()->enqueueMessage(JText::_('COM_SECURITYCHECKPRO_REACHED_ONLINE_FILES'),'error');
			$error = true;
		}
	} else {
		JFactory::getApplication()->enqueueMessage(JText::_('COM_SECURITYCHECKPRO_NO_FILES_SELECTED'),'error');	
		$error = true;
	}
	
	return $error;
	
}

/* Chequea hashes contra el servicio OPWAST Metascan online */
function online_check_hashes()
{
	
	// Inicializamos las variables
	$this->analized_keys_array = array();
	$error = false;
	
	// Obtenemos la API key
	$params = JComponentHelper::getParams('com_securitycheckpro');
	$apikey = $params->get('opswat_key','');
	
	// Creamos el objeto JInput para obtener las variables del formulario
	$jinput = JFactory::getApplication()->input;
	
	// Obtenemos las rutas de los ficheros a analizar
	$paths = $jinput->get('malwarescan_status_table',null,'array');
			
	if ( !empty($paths) ) {	
		
		// Creamos el nuevo objeto query
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		
		// Cargamos el contenido del fichero de archivos sospechosos
		$malwarescan_data = $this->loadStack("malwarescan","malwarescan");
		// Creamos un array de rutas para modificar los elementos que hayan sido escaneados
		$array_paths = array_map( function ($element) { return $element['path']; },$malwarescan_data );
		
		// Nmero de archivos escaneados en la ltima hora
		$this->analized_hashes_last_hour = $this->get_online_analyzed_values("hashes");
		
		// Chequeamos si sobrepasamos el lmite de hashes a analizar por hora (1000)
		if ( ($this->analized_hashes_last_hour) + (count($paths)) <= 1000 ) {
			foreach($paths as $path) {		
				// Buscamos la clave del array a modificar
				$array_key = array_search($path,$array_paths);
				// Almacenamos el valor encontrado para utilizarlo posteriormente
				array_push($this->analized_keys_array,$array_key);					
			}
			
			// Preguntamos directamente al servicio online por cada valor hash seleccionado.
			$this->look_for_results($apikey,$malwarescan_data,"hashes");	
		} else {
			JFactory::getApplication()->enqueueMessage(JText::_('COM_SECURITYCHECKPRO_REACHED_ONLINE_FILES'),'error');
			$error = true;
		}
	
	} else {
		JFactory::getApplication()->enqueueMessage(JText::_('COM_SECURITYCHECKPRO_NO_FILES_SELECTED'),'error');	
		$error = true;
	}
	
	return $error;
		
}

/* Funcin que obtiene el nmero de archivos o hashes escaneados online durante la ltima hora */
function get_online_analyzed_values($type){

	// Inicializamos las variables
	$analyzed = 0;
	
	// Import Securitycheckpros model
	JLoader::import('joomla.application.component.model');
	JLoader::import('filemanager', JPATH_ADMINISTRATOR . DIRECTORY_SEPARATOR . 'components' . DIRECTORY_SEPARATOR. 'com_securitycheckpro' . DIRECTORY_SEPARATOR . 'models');
	$model = JModelLegacy::getInstance( 'filemanager', 'SecuritycheckprosModel');
	
	$last_check = new DateTime(date('Y-m-d',strtotime($this->get_campo_filemanager('last_online_check_malwarescan'))));
	$now = new DateTime(date('Y-m-d',strtotime($model->currentDateTime_func())));
					
	// Calculamos las horas que han pasado desde el ltimo chequeo
	(int) $interval = $now->diff($last_check)->format("%h");
		
	// Si ha pasado una hora o ms desde el ltimo escaneo, inicializamos el valor almacenado en la BBDD. De lo contrario devolvemos el valor almacenado en la BBDD.
	if ( $interval >= 1 ) {
		switch ( $type ) {
			case "files":
				$this->set_campo_filemanager('online_checked_files',0);
				break;
			case "hashes":
				$this->set_campo_filemanager('online_checked_hashes',0);
				break;
		}
	} else {
		switch ( $type ) {
			case "files":
				$analyzed = $this->get_campo_filemanager('online_checked_files');
				break;
			case "hashes":
				$analyzed = $this->get_campo_filemanager('online_checked_hashes');
				break;
		}
		
	}
	
	return $analyzed;
}

/* Funcin que obtiene el resultado de cada uno de los archivos o hashes escaneados online */
private function look_for_results($apikey,$malwarescan_data,$opcion) {

	/* Cargamos el lenguaje del sitio */
	$lang = JFactory::getLanguage();
	$lang->load('com_securitycheckpro',JPATH_ADMINISTRATOR);
	
	// Inicializamos las variables
	switch ( $opcion ) {
			case "files":
				$file_analysis_result = "#<?php die('Forbidden.'); ?>" . PHP_EOL . "<h3>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_FILE_SCAN') . "</h3>" . PHP_EOL;
				break;
			case "hashes":
				$file_analysis_result = "#<?php die('Forbidden.'); ?>" . PHP_EOL . "<h3>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_HASHES_SCAN') . "</h3>" . PHP_EOL;
				break;
	}
	$threats_found = 0;
	
	foreach ($this->analized_keys_array as $array_key) {
	
		switch ( $opcion ) {
			case "files":
				//Config.
				// Cuando el escaneo del archivo se est realizando, hemos de hacer consultas al servidor proporcionado por la clave 'rest_ip'
				$api        = 'https://' . $malwarescan_data[$array_key]['rest_ip'] . '/file/' .$malwarescan_data[$array_key]['data_id'];
				
				//Build headers array.
				$headers = array(
					'apikey: '.$apikey
				);

				//Build options array.
				$options = array(
					CURLOPT_URL     => $api,
					CURLOPT_HTTPHEADER  => $headers,
					CURLOPT_RETURNTRANSFER  => true,
					CURLOPT_SSL_VERIFYPEER  => false
				);

				$response = "";
				//Init & execute API call.
				$ch = curl_init();
				curl_setopt_array($ch, $options);
				
				do {
					$response = json_decode(curl_exec($ch), true);
				}
				while ($response["scan_results"]["progress_percentage"] != 100);
				
				// Una vez finalizado el escaneo, hacemos una peticin ms (esta vez al servicio en scan.metascan-online.com) para obtener el resultado
				$api        = 'https://scan.metascan-online.com/v2/file/' .$malwarescan_data[$array_key]['data_id'];
				
				//Build headers array.
				$headers = array(
					'apikey: '.$apikey
				);

				//Build options array.
				$options = array(
					CURLOPT_URL     => $api,
					CURLOPT_HTTPHEADER  => $headers,
					CURLOPT_RETURNTRANSFER  => true,
					CURLOPT_SSL_VERIFYPEER  => false
				);

				$response = "";
				//Init & execute API call.
				$ch = curl_init();
				curl_setopt_array($ch, $options);
				
				$response = json_decode(curl_exec($ch), true);
				break;
			case "hashes":
				// Establecemos el valor del hash en la variable 'api'
				$api = 'https://hashlookup.metascan-online.com/v2/hash/' . $malwarescan_data[$array_key]['sha1_value'];
							
				// Build headers array.
				$headers = array(
					'apikey: '.$apikey
				);

				// Build options array.
				$options = array(
					CURLOPT_URL     => $api,
					CURLOPT_HTTPHEADER  => $headers,
					CURLOPT_RETURNTRANSFER  => true,
					CURLOPT_SSL_VERIFYPEER  => false
				);

				// Init & execute API call.
				$ch = curl_init();
				curl_setopt_array($ch, $options);
				
				$response = json_decode(curl_exec($ch), true);
				
				// Incrementamos el valor de la variable de archivos analizados
				$this->analized_hashes_last_hour++;				
				
				break;
		}
		
		if ( is_array($response) ) {
			// Guardamos el resultado del escaneo online
			$malwarescan_data[$array_key]['online_check'] = $response["scan_results"]["scan_all_result_i"];
		
			// Actualizamos la variable de amenazas encontradas si es que se han encontrado
			if ( ($response["scan_results"]["scan_all_result_i"] == 1) || ($response["scan_results"]["scan_all_result_i"] == 2) ) {
				$threats_found++;
			}
			
			// Aadimos el resultado a la variable que ser volcada en el fichero de resultados
			$file_analysis_result .= $this->format_data($response);
		} else {
			// Guardamos el resultado del escaneo online ( le asignamos el valor '15' )
			$malwarescan_data[$array_key]['online_check'] = 15;
			
			// Aadimos el resultado a la variable que ser volcada en el fichero de resultados. Pasamos los datos del fichero ya que el hash no se ha encontrado en la BBDD
			$file_analysis_result .= $this->format_data($response,true,$malwarescan_data[$array_key]);
		}
	}
		
	// Si la opcin seleccionada es el escaneo de hashes, actualizamos las variables correspondientes en la bbdd.
	if ( $opcion == "hashes" ) {
		// Actualizamos los valores de los campos relacionados con el analisis online
		$this->set_campo_filemanager('online_checked_hashes',$this->analized_hashes_last_hour);
		$this->set_campo_filemanager('last_online_check_malwarescan',date('Y-m-d H:i:s'));	
	}
	
	// Borramos el fichero del escaneo anterior...
	$delete_malwarescan_file = JFile::delete($this->folder_path.$this->malwarescan_name);
			
	// ... y almacenamos el nuevo contenido
	try {
		$content_malwarescan = utf8_encode(json_encode(array('files_folders'	=> $malwarescan_data)));
		$content_malwarescan = "#<?php die('Forbidden.'); ?>" . PHP_EOL . $content_malwarescan;
		$result_malwarescan = JFile::write($this->folder_path.DIRECTORY_SEPARATOR.$this->malwarescan_name, $content_malwarescan);
	} catch (Exception $e) {
				
	}
	
	// Comprobamos si hay algo que escribir
	if ( !is_null($file_analysis_result) ) {
		// Escribimos el contenido del buffer en un fichero
		$status = $this->write_file($file_analysis_result,$threats_found,count($this->analized_keys_array));
	}
}

/* Funcin que formatea los datos de entrada (en un array) para adaptarlos al del fichero */
private function format_data($response, $not_found = false, $file_data = null) {
	
	/* Cargamos el lenguaje del sitio */
	$lang = JFactory::getLanguage();
	$lang->load('com_securitycheckpro',JPATH_ADMINISTRATOR);

	// Inicializamos las variables
	$data = null;
	$scan_result = '';
	
	// El hash se ha encontrado en la BBDD
	if ( !$not_found ) {
	
		$data = "<h4>" . $response["file_info"]["display_name"] . "</h4>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_UPLOAD_TIMESTAMP') . ": " . $response["file_info"]["upload_timestamp"] . "</p>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_FILE_SIZE') . ": " . $response["file_info"]["file_size"] . "</p>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_DATA_ID') . ": " . $response["data_id"] . "</p>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SHA256') . ": " . $response["file_info"]["sha256"] . "</p>" . PHP_EOL;
			
		switch ($response["scan_results"]["scan_all_result_i"]) {
			case 0:
				$scan_result = "<span style=\"color: #008000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_CLEAN') . "</span></strong>";
				break;
			case 1:
				$scan_result = "<span style=\"color: #FF0000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_INFECTED') . "</span></strong>";
				break;
			case 2:
				$scan_result = "<span style=\"color: #FF4000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_SUSPICIOUS') . "</span></strong>";
				break;
			case 3:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_FAILED_TO_SCAN') . "</span></strong>";
				break;
			case 4:
				$scan_result = "<span style=\"color: #000000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_CLEANED') . "</span></strong>";
				break;
			case 5:
				$scan_result = "<span style=\"color: #000000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_UNKNOW') . "</span></strong>";
				break;
			case 6:
				$scan_result = "<span style=\"color: #000000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_QUARANTINED') . "</span></strong>";
				break;
			case 7:
				$scan_result = "<span style=\"color: #000000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_SKIPPED_CLEAN') . "</span></strong>";
				break;
			case 8:
				$scan_result = "<span style=\"color: #000000;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_SKIPPED_DIRTY') . "</span></strong>";
				break;
			case 9:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_EXCEEDED_DEPTH') . "</span></strong>";
			case 10:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_NOT_SCANNED') . "</span></strong>";
				break;
			case 11:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_ABORTED') . "</span></strong>";
				break;
			case 12:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_ENCRYPTED') . "</span></strong>";
				break;
			case 13:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_EXCEEDED_SIZE') . "</span></strong>";
				break;
			case 14:
				$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_EXCEEDED_FILE_NUMBER') . "</span></strong>";
				break;
		}
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS') . ": " . $scan_result . "</p>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_TOTAL_AVS') . ": " . $response["scan_results"]["total_avs"] . "</p>" . PHP_EOL . PHP_EOL;
			
		// Actualizamos la variable de amenazas encontradas si es que se han encontrado
		if ( $response["scan_results"]["scan_all_result_i"] == 1 ) {
			$data .= "<h5 style=\"color: #2E64FE;\">" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_AVS_RESULT') . "</h5>" . PHP_EOL;
			$data .= "<table border=\"1\">" . PHP_EOL;
			$data .= "<thead>" . PHP_EOL;
			$data .= "<tr>" . PHP_EOL;
			$data .= "<th>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_AV_ENGINE') . "</th>" . PHP_EOL;
			$data .= "<th>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_DEF_TIME') . "</th>" . PHP_EOL;
			$data .= "<th>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_TIME') . "</th>" . PHP_EOL;
			$data .= "<th>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_THREAT_FOUND') . "</th>" . PHP_EOL;
			$data .= "</tr>";
			$data .= "</thead>";
			$data .= "</tbody>";		
			
			// Extraemos los nombres de los motores de antivirus usados
			$av_engines =  array_keys($response["scan_results"]['scan_details']);
			$indice = 0;
			foreach ($response["scan_results"]['scan_details'] as $av){
				$data .= "<tr>" . PHP_EOL;
				if ( empty($av['threat_found']) ) {
					$data .= "<td style=\"text-align: center; vertical-align: middle;\">" . $av_engines[$indice] . "</td>" . PHP_EOL;
				} else {
					$data .= "<td style=\"text-align: center; vertical-align: middle;\"><font color=#5858FA>" . $av_engines[$indice] . "</font></td>" . PHP_EOL;
				}
				$data .= "<td style=\"text-align: center; vertical-align: middle;\">" . $av['def_time'] . "</td>" . PHP_EOL;
				$data .= "<td style=\"text-align: center; vertical-align: middle;\">" . $av['scan_time'] . "</td>" . PHP_EOL;
				if ( empty($av['threat_found']) ) {
					$data .= "<td style=\"text-align: center; vertical-align: middle;\">" . $av['threat_found'] . "</td>" . PHP_EOL;
				} else {
					$data .= "<td style=\"text-align: center; vertical-align: middle;\"><font color=red>" . $av['threat_found'] . "</font></td>" . PHP_EOL;
				}
				$data .= "</tr>" . PHP_EOL;
				$indice++;
			}
			$data .= "</tbody>";
			$data .= "</table>" . PHP_EOL;
		}
	} else {
		$data = "<h4>" . $file_data["path"] . "</h4>" . PHP_EOL;
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SHA256') . ": " . $file_data["sha1_value"] . "</p>" . PHP_EOL;
		$scan_result = "<span style=\"color: #61380B;\"><strong>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS_HASH_NOT_FOUND') . "</span></strong>";
		$data .= "<p>" . $lang->_('COM_SECURITYCHECKPRO_MALWARESCAN_SCAN_RESULTS') . ": " . $scan_result . "</p>" . PHP_EOL;
	}
	
	return $data;

}

/* Funcin que guarda el resultado del escaneo en un fichero y actualiza la bbdd */
private function write_file($file_analysis_result,$threats,$files_checked) {

	// Nombre del fichero
	$filename = $this->generateKey();
	
	// Comprobamos si hay que borrar archivos por alcanzar el lmite establecido
	$this->check_logs_stored();
	
	// Escribrimos el fichero
	try {
		$file_result = JFile::write($this->folder_path.DIRECTORY_SEPARATOR.$filename, $file_analysis_result);
	} catch (Exception $e) {
				
	}
	
	// Actualizamos la bbdd con la informacin de este nuevo fichero
	if ( $file_result ) {
		$db = JFactory::getDBO();
			
		// Sanitizamos la entrada
		$threats = $db->escape($threats);
			
		$sql = "INSERT INTO `#__securitycheckpro_online_checks` ( `filename`, `files_checked`, `threats_found`, `scan_date` ) VALUES ('{$filename}', '{$files_checked}', '{$threats}', now())";
		$db->setQuery($sql);
		$db->query();
	}

}

// Chequeamos si ha pasado ms de una hora desde el ltimo escaneo online para inicializar la variable que la controla
private function check_last_onlinecheck() {

	// ltimo escaneo
	$last_check = new DateTime($this->get_campo_filemanager("last_online_check_malwarescan"));

	// Ahora
	$now = new DateTime(date('Y-m-d H:i:s'));

	// Diferencia
	$difference = $last_check->diff($now);

	// Si ha pasado ms de una hora, inicializamos la variable
	if ( (($difference->h)>=1) || ( (($difference->h)==0) && (($difference->d)>=1) ) ) {
		$this->set_campo_filemanager("online_checked_files",0);
	}

}

private function check_logs_stored() {

	// Inicializamos las variables
	$files_deleted = 0;
	
	// Consultamos los valores de configuracin
	$params = JComponentHelper::getParams('com_securitycheckpro');
	(int) $log_files_to_store = $params->get('log_files_stored',5);
	
	$db = JFactory::getDBO();
			
	$sql = "SELECT COUNT(*) FROM `#__securitycheckpro_online_checks`";
	$db->setQuery($sql);
	(int) $logs_stored = $db->loadResult();
		
	// Si se ha sobrepasado el lmite de archivos que se deben guardar, los borramos del directorio y de la bbdd
	if ( $logs_stored >= $log_files_to_store ) {
	
		// Extraemos el array de ficheros almacenados en orden descendente
		$query = $db->getQuery(true)
		->select(array('filename'))
		->from($db->quoteName('#__securitycheckpro_online_checks'))
		->order('scan_date DESC');
		$db->setQuery($query);
		$filenames = $db->loadRowList();


				
		// Inicializamos el ndice para recorrer el array
		$indice = 0;
		foreach ( $filenames as $filename ) {
			if ( $indice >= ($log_files_to_store-1) ) {
				// Borramos el fichero
				$delete_file = JFile::delete($this->folder_path.$filename[0]);
				// Si el fichero se ha borrado actualizamos la bbdd
				if ( $delete_file ) {
					$query = $db->getQuery(true)
						->delete($db->quoteName('#__securitycheckpro_online_checks'))
						->where($db->quoteName('filename').' = '.$db->quote($filename[0]));
					$db->setQuery($query);
					$db->query();
				}
				$files_deleted++;
			}
			$indice++;
		}
		JFactory::getApplication()->enqueueMessage(JText::sprintf('COM_SECURITYCHECKPRO_DELETED_OLD_FILES',$files_deleted));				
	}	


}

}